/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No 
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all 
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM 
* EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES 
* SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS 
* SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of 
* this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer 
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : resetprg.c
* Device(s)    : RX130
* Description  : Defines post-reset routines that are used to configure the MCU prior to the main program starting. 
*                This is where the program counter starts on power-up or reset.
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 01.12.2015 1.00     First release
*         : 15.05.2017 1.01     Deleted unnecessary comments.
*                               Changed the method of setting the following registers.
*                                - Main Clock Oscillator Forced Oscillation Control Register (MOFCR.MOSEL)
*                                - Main Clock Oscillator Wait Control Register (MOSCWTCR)
*                               Changed the method of Wait for at least five cycles (about 153 s) of the sub-clock.
*                               Deleted the following unnecessary branch condition in lpt_clock_source_select function.
*                                - BSP_CFG_LPT_CLOCK_SOURCE == 2
*                               Added the setting of IWDT.IWDTCSTPR.BIT.SLCSTP.
*                               Deleted the Change_PSW_PM_to_UserMode function.
*         : 01.11.2017 2.00     Added the bsp startup module disable function.
*                               Added the following build condition in lpt_clock_source_select function.
*                               - BSP_CFG_LPT_CLOCK_SOURCE == 2
*         : 27.07.2018 2.01     Added the comment to for statement.
*                               Added the comment to while statement.
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes   <System Includes> , "Project Includes"
***********************************************************************************************************************/
/* Defines MCU configuration functions used in this file */
#include    <_h_c_lib.h>

/* This macro is here so that the stack will be declared here. This is used to prevent multiplication of stack size. */
#define     BSP_DECLARE_STACK
/* Define the target platform */
#include    "platform.h"

/* When using the user startup program, disable the following code. */
#if (BSP_CFG_STARTUP_DISABLE == 0)

/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/
/* If the user chooses only 1 stack then the 'U' bit will not be set and the CPU will always use the interrupt stack. */
#if (BSP_CFG_USER_STACK_ENABLE == 1)
    #define PSW_init  (0x00030000)
#else
    #define PSW_init  (0x00010000)
#endif

/* IWDTCLK Oscillation stabilization time is more than 50us */
/* When the system clock is the sub-clock, warning will occur but there is no problem.
   Because the calculation result of this definition is zero. */
#define IWDTCLK_STABILIZE_LOOP_CNT    ((uint32_t)(0.00005f * BSP_ICLK_HZ / 10))


/***********************************************************************************************************************
Pre-processor Directives
***********************************************************************************************************************/
/* Set this as the entry point from a power-on reset */
#pragma entry PowerON_Reset_PC

/***********************************************************************************************************************
External function Prototypes
***********************************************************************************************************************/
/* Functions to setup I/O library */
extern void _INIT_IOLIB(void);
extern void _CLOSEALL(void);

/* Functions to use global class object */
/* Sections C$INIT and C$END will be generated */
extern void _CALL_INIT(void);
extern void _CALL_END(void);

#if BSP_CFG_USER_WARM_START_CALLBACK_PRE_INITC_ENABLED != 0
/* If user is requesting warm start callback functions then these are the prototypes. */
void BSP_CFG_USER_WARM_START_PRE_C_FUNCTION(void);
#endif

#if BSP_CFG_USER_WARM_START_CALLBACK_POST_INITC_ENABLED != 0
/* If user is requesting warm start callback functions then these are the prototypes. */
void BSP_CFG_USER_WARM_START_POST_C_FUNCTION(void);
#endif

/***********************************************************************************************************************
Private global variables and functions
***********************************************************************************************************************/
/* Power-on reset function declaration */
void PowerON_Reset_PC(void);

/* Main program function declaration */
void main(void);
void abort(void);
static void operating_frequency_set(void);
static void clock_source_select(void);
static void lpt_clock_source_select(void);


/***********************************************************************************************************************
* Function name: PowerON_Reset_PC
* Description  : This function is the MCU's entry point from a power-on reset.
*                The following steps are taken in the startup code:
*                1. The User Stack Pointer (USP) and Interrupt Stack Pointer (ISP) are both set immediately after entry 
*                   to this function. The USP and ISP stack sizes are set in the file bsp_config.h.
*                   Default sizes are USP=1K and ISP=256.
*                2. The interrupt vector base register is set to point to the beginning of the relocatable interrupt 
*                   vector table.
*                3. The MCU operating frequency is set by operating_frequency_set.
*                4. Calls are made to functions to setup the C runtime environment which involves initializing all 
*                   initialed data, zeroing all uninitialized variables, and configuring STDIO if used
*                   (calls to _INITSCT and _INIT_IOLIB).
*                5. Board-specific hardware setup, including configuring I/O pins on the MCU, in hardware_setup.
*                6. Global interrupts are enabled by setting the I bit in the Program Status Word (PSW), and the stack 
*                   is switched from the ISP to the USP.  The initial Interrupt Priority Level is set to zero, enabling 
*                   any interrupts with a priority greater than zero to be serviced.
*                7. The processor is optionally switched to user mode.  To run in user mode, set the macro 
*                   BSP_CFG_RUN_IN_USER_MODE above to a 1.
*                8. The bus error interrupt is enabled to catch any accesses to invalid or reserved areas of memory.
*
*                Once this initialization is complete, the user's main() function is called.  It should not return.
* Arguments    : none
* Return value : none
***********************************************************************************************************************/
void PowerON_Reset_PC(void)
{
    /* Stack pointers are setup prior to calling this function - see comments above */    
    
    /* Initialize the Interrupt Table Register */
#if __RENESAS_VERSION__ >= 0x01010000
    set_intb((void *)__sectop("C$VECT"));
#else
    set_intb((unsigned long)__sectop("C$VECT"));
#endif

    /* Switch to high-speed operation */
    operating_frequency_set();
    lpt_clock_source_select();


    /* If the warm start Pre C runtime callback is enabled, then call it. */
#if BSP_CFG_USER_WARM_START_CALLBACK_PRE_INITC_ENABLED == 1
     BSP_CFG_USER_WARM_START_PRE_C_FUNCTION();
#endif

    /* Initialize C runtime environment */
    _INITSCT();

    /* If the warm start Post C runtime callback is enabled, then call it. */
#if BSP_CFG_USER_WARM_START_CALLBACK_POST_INITC_ENABLED == 1
     BSP_CFG_USER_WARM_START_POST_C_FUNCTION();
#endif

#if BSP_CFG_IO_LIB_ENABLE == 1
    /* Comment this out if not using I/O lib */
    _INIT_IOLIB();
#endif

    /* Initialize MCU interrupt callbacks. */
    bsp_interrupt_open();

    /* Initialize register protection functionality. */
    bsp_register_protect_open();

    /* Configure the MCU and board hardware */
    hardware_setup();

#ifdef CPPAPP
    /* Use global class object */
    _CALL_INIT();
#endif

    /* Change the MCU's user mode from supervisor to user */
    nop();
    set_psw(PSW_init);
#if BSP_CFG_RUN_IN_USER_MODE==1
    /* Use chg_pmusr() intrinsic if possible. */
    chg_pmusr() ;
#endif

    /* Enable the bus error interrupt to catch accesses to illegal/reserved areas of memory */
    R_BSP_InterruptControl(BSP_INT_SRC_BUS_ERROR, BSP_INT_CMD_INTERRUPT_ENABLE, FIT_NO_PTR);

    /* Call the main program function (should not return) */
    main();
    
#if BSP_CFG_IO_LIB_ENABLE == 1
    /* Comment this out if not using I/O lib - cleans up open files */
    _CLOSEALL();
#endif

#ifdef CPPAPP
    /* Use global class object */
    _CALL_END();
#endif

    /* Infinite loop is intended here. */    
    /* WAIT_LOOP */
    while(1)
    {
        /* Infinite loop. Put a breakpoint here if you want to catch an exit of main(). */
    }
}

/***********************************************************************************************************************
* Function name: abort
* Description  : Abort (mandatory for C++)
* Arguments    : none
* Return value : none
***********************************************************************************************************************/
void abort(void)
{
#if BSP_CFG_IO_LIB_ENABLE == 1
    /* Comment this out if not using I/O lib - cleans up open files */
    _CLOSEALL();
#endif

#ifdef CPPAPP
    /* Use global class object */
    _CALL_END();
#endif

    /* Infinite loop is intended here. */    
    /* WAIT_LOOP */
    while(1)
    {
        /* Infinite loop. Put a breakpoint here if you want to catch an exit of main(). */
    }
}

/***********************************************************************************************************************
* Function name: operating_frequency_set
* Description  : Configures the clock settings for each of the device clocks
* Arguments    : none
* Return value : none
***********************************************************************************************************************/
static void operating_frequency_set (void)
{
    /* Used for constructing value to write to SCKCR and CKOCR registers. */
    uint32_t temp_clock = 0;
    
    /* 
    Default settings:
    Clock Description              Frequency
    ----------------------------------------
    Input Clock Frequency............  8  MHz
    PLL frequency (/2 x8)............  32 MHz
    Internal Clock Frequency.........  32 MHz
    Peripheral Clock B Frequency...... 32 MHz
    Peripheral Clock D Frequency...... 32 MHz
    Flash IF Clock Frequency.......... 32 MHz */

    /* Protect off. */
    SYSTEM.PRCR.WORD = 0xA50F;
    
    /* Select the clock based upon user's choice. */
    clock_source_select();

    /* Figure out setting for FCK bits. */
#if   BSP_CFG_FCK_DIV == 1
    /* Do nothing since FCK bits should be 0. */
#elif BSP_CFG_FCK_DIV == 2
    temp_clock |= 0x10000000;
#elif BSP_CFG_FCK_DIV == 4
    temp_clock |= 0x20000000;
#elif BSP_CFG_FCK_DIV == 8
    temp_clock |= 0x30000000;
#elif BSP_CFG_FCK_DIV == 16
    temp_clock |= 0x40000000;
#elif BSP_CFG_FCK_DIV == 32
    temp_clock |= 0x50000000;
#elif BSP_CFG_FCK_DIV == 64
    temp_clock |= 0x60000000;
#else
    #error "Error! Invalid setting for BSP_CFG_FCK_DIV in r_bsp_config.h"
#endif

    /* Figure out setting for ICK bits. */
#if   BSP_CFG_ICK_DIV == 1
    /* Do nothing since ICK bits should be 0. */
#elif BSP_CFG_ICK_DIV == 2
    temp_clock |= 0x01000000;
#elif BSP_CFG_ICK_DIV == 4
    temp_clock |= 0x02000000;
#elif BSP_CFG_ICK_DIV == 8
    temp_clock |= 0x03000000;
#elif BSP_CFG_ICK_DIV == 16
    temp_clock |= 0x04000000;
#elif BSP_CFG_ICK_DIV == 32
    temp_clock |= 0x05000000;
#elif BSP_CFG_ICK_DIV == 64
    temp_clock |= 0x06000000;
#else
    #error "Error! Invalid setting for BSP_CFG_ICK_DIV in r_bsp_config.h"
#endif

    /* Figure out setting for PCKB bits. */
#if   BSP_CFG_PCKB_DIV == 1
    /* Do nothing since PCKB bits should be 0. */
#elif BSP_CFG_PCKB_DIV == 2
    temp_clock |= 0x00000100;
#elif BSP_CFG_PCKB_DIV == 4
    temp_clock |= 0x00000200;
#elif BSP_CFG_PCKB_DIV == 8
    temp_clock |= 0x00000300;
#elif BSP_CFG_PCKB_DIV == 16
    temp_clock |= 0x00000400;
#elif BSP_CFG_PCKB_DIV == 32
    temp_clock |= 0x00000500;
#elif BSP_CFG_PCKB_DIV == 64
    temp_clock |= 0x00000600;
#else
    #error "Error! Invalid setting for BSP_CFG_PCKB_DIV in r_bsp_config.h"
#endif

    /* Figure out setting for PCKD bits. */
#if   BSP_CFG_PCKD_DIV == 1
    /* Do nothing since PCKD bits should be 0. */
#elif BSP_CFG_PCKD_DIV == 2
    temp_clock |= 0x00000001;
#elif BSP_CFG_PCKD_DIV == 4
    temp_clock |= 0x00000002;
#elif BSP_CFG_PCKD_DIV == 8
    temp_clock |= 0x00000003;
#elif BSP_CFG_PCKD_DIV == 16
    temp_clock |= 0x00000004;
#elif BSP_CFG_PCKD_DIV == 32
    temp_clock |= 0x00000005;
#elif BSP_CFG_PCKD_DIV == 64
    temp_clock |= 0x00000006;
#else
    #error "Error! Invalid setting for BSP_CFG_PCKD_DIV in r_bsp_config.h"
#endif

    /* Set SCKCR register. */
    SYSTEM.SCKCR.LONG = temp_clock;
    /* WAIT_LOOP */
    while (temp_clock != SYSTEM.SCKCR.LONG)
    {
         /* Confirm that the written value can be read correctly. */
    }

    /* Choose clock source. Default for r_bsp_config.h is PLL. */
    SYSTEM.SCKCR3.WORD = ((uint16_t)BSP_CFG_CLOCK_SOURCE) << 8;
    /* WAIT_LOOP */
    while ((((uint16_t)BSP_CFG_CLOCK_SOURCE) << 8) != SYSTEM.SCKCR3.WORD)
    {
        /* Confirm that the written value can be read correctly. */
    }

#if (BSP_CFG_CLOCK_SOURCE != 0)
    /* We can now turn LOCO off since it is not going to be used. */
    SYSTEM.LOCOCR.BYTE = 0x01;
#endif

    /* Protect on. */
    SYSTEM.PRCR.WORD = 0xA500;
}

/***********************************************************************************************************************
* Function name: clock_source_select
* Description  : Enables and disables clocks as chosen by the user. If a clock other than LOCO or HOCO is already
*                running when this function is called then that usually means a bootloader was run beforehand and set
*                up the clocks already.
* Arguments    : none
* Return value : none
***********************************************************************************************************************/
static void clock_source_select (void)
{
    /* Declared volatile for software delay purposes. */
    volatile uint32_t i;

#if (BSP_CFG_CLOCK_SOURCE != 3) && (BSP_CFG_LPT_CLOCK_SOURCE != 0)
    volatile uint8_t dummy;
#endif

    /* Set to High-speed operating mode if ICLK is > 12MHz. */
    if (BSP_ICLK_HZ > BSP_MIDDLE_SPEED_MAX_FREQUENCY)
    {
        /* WAIT_LOOP */
        while(1 == SYSTEM.OPCCR.BIT.OPCMTSF)
        {
            /* Wait for transition to finish. */
        }

        SYSTEM.OPCCR.BYTE = 0x00;   // set to high-speed mode

        /* WAIT_LOOP */
        while(1 == SYSTEM.OPCCR.BIT.OPCMTSF)
        {
            /* Wait for transition to finish. */
        }

    }

    /* At this time the MCU is still running on the 4MHz LOCO. */

#if (BSP_CFG_CLOCK_SOURCE == 1)
    /* HOCO is chosen. Start it operating if it is not already operating. */
    if (1 == SYSTEM.HOCOCR.BIT.HCSTP)
    {
        /* Start HOCO operating. */
        SYSTEM.HOCOCR.BYTE = 0x00;
    }

    /* WAIT_LOOP */
    while (0 == SYSTEM.OSCOVFSR.BIT.HCOVF)
    {
        /* Make sure clock has stabilized. */
    }
#else
    /* HOCO is not chosen. If HOCO is already operating, it doesn't stop.  */
#endif

#if (BSP_CFG_CLOCK_SOURCE == 2)
    /* Main clock oscillator is chosen. Start it operating if it is not already operating. If it was already operating*/
    if (1 == SYSTEM.MOSCCR.BIT.MOSTP)
    {
        /* If the main oscillator is > 10MHz, and VCC is >= 2.4V then the main clock oscillator
           forced oscillation control register (MOFCR) must be changed. */
        if ((BSP_CFG_MCU_VCC_MV >= 2400) && (BSP_CFG_XTAL_HZ > 10000000))
        {
            /* 10 - 20MHz. */
            SYSTEM.MOFCR.BIT.MODRV21 = 1;
        }
        else
        {
            /* 1 - 10MHz. */
            SYSTEM.MOFCR.BIT.MODRV21 = 0;
        }

        /* Set the oscillation source of the main clock oscillator. */
        SYSTEM.MOFCR.BIT.MOSEL = BSP_CFG_MAIN_CLOCK_SOURCE;

        /* Set the oscillation stabilization wait time of the main clock oscillator. */
        #if (BSP_CFG_MAIN_CLOCK_SOURCE == 0) /* Resonator */
            SYSTEM.MOSCWTCR.BYTE = BSP_CFG_MOSC_WAIT_TIME;
        #elif (BSP_CFG_MAIN_CLOCK_SOURCE == 1) /* External oscillator input */
            SYSTEM.MOSCWTCR.BYTE = 0x00;
        #else
            #error "Error! Invalid setting for BSP_CFG_MAIN_CLOCK_SOURCE in r_bsp_config.h"
        #endif 

        /* Set the main clock to operating. */
        SYSTEM.MOSCCR.BYTE = 0x00;
    }

    /* WAIT_LOOP */
    while (0 == SYSTEM.OSCOVFSR.BIT.MOOVF)
    {
        /* Make sure clock has stabilized. */
    }
#else
    /* Set the main clock to stopped. */
    SYSTEM.MOSCCR.BYTE = 0x01;          
#endif

#if (BSP_CFG_CLOCK_SOURCE == 3) || (BSP_CFG_LPT_CLOCK_SOURCE == 0)

    SYSTEM.SOSCCR.BYTE = 0x01;      /* Make sure sub-clock is initially stopped */
    /* WAIT_LOOP */
    while (SYSTEM.SOSCCR.BYTE != 0x01)
    {
        /* wait for bit to change */
    }

    RTC.RCR3.BIT.RTCEN = 0;         /* Also set the RTC Sub-Clock disable */
    /* WAIT_LOOP */
    while (RTC.RCR3.BIT.RTCEN != 0)
    {
        /* wait for bit to change */
    }

    /* wait for 5 sub-clock cycles (153us): measurement result is approx. 176us */
    R_BSP_SoftwareDelay(176, BSP_DELAY_MICROSECS);    /* 153us * 4.56 / 4.00 (LOCO max) */

    RTC.RCR3.BIT.RTCDV = 0x01;      /* Init the Sub-Clock Oscillator Drive Capacity = Low CL */
    /* WAIT_LOOP */
    while (RTC.RCR3.BIT.RTCDV != 0x01)
    {
        /* wait for bits to change */
    }

    SYSTEM.SOSCCR.BYTE = 0x00;      /* Start sub-clock */
    /* WAIT_LOOP */
    while (SYSTEM.SOSCCR.BYTE != 0x00)
    {
        /* wait for bit to change */
    }


    /* Wait for the oscillation stabilization time of the sub-clock. (1.3 seconds.)*/
    R_BSP_SoftwareDelay(1482, BSP_DELAY_MILLISECS);    /* 1.3s * 4.56 / 4.00 (LOCO max) */


    RTC.RCR3.BIT.RTCEN = 0x01;      /* Also set the RTC Sub-Clock enable */
    /* WAIT_LOOP */
    while (RTC.RCR3.BIT.RTCEN != 0x01)
    {
        /* wait for bit to change */
    }
#else

    /* Wait for the oscillation stabilization time of the sub-clock. (1.3 seconds.)*/
    R_BSP_SoftwareDelay(1482, BSP_DELAY_MILLISECS);    /* 1.3s * 4.56 / 4.00 (LOCO max) */

    /* Set the sub-clock to stopped. */
    SYSTEM.SOSCCR.BYTE = 0x01;
    /* WAIT_LOOP */
    while (0x01 != SYSTEM.SOSCCR.BYTE)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* Set the sub-clock to stopped. */
    RTC.RCR3.BIT.RTCEN = 0;

    /* dummy read three times */
    /* WAIT_LOOP */
    for (i = 0; i < 3; i++)
    {
        dummy = RTC.RCR3.BIT.RTCEN;
    }

    /* WAIT_LOOP */
    while (0 != RTC.RCR3.BIT.RTCEN)
    {
        /* Confirm that the written */
    }

#endif

#if (BSP_CFG_CLOCK_SOURCE == 4)
    /* PLL is chosen. Start it operating if it is not already. Must start main clock as well since PLL uses it. */

    if (1 == SYSTEM.PLLCR2.BIT.PLLEN)
    {
        /* If the main oscillator is > 10MHz, and VCC is >= 2.4V then the main clock oscillator
           forced oscillation control register (MOFCR) must be changed. */
        if ((BSP_CFG_MCU_VCC_MV >= 2400) && (BSP_CFG_XTAL_HZ > 10000000))
        {
            /* 10 - 20MHz. */
            SYSTEM.MOFCR.BIT.MODRV21 = 1;
        }
        else
        {
            /* 1 - 10MHz. */
            SYSTEM.MOFCR.BIT.MODRV21 = 0;
        }

        /* Set the oscillation source of the main clock oscillator. */
        SYSTEM.MOFCR.BIT.MOSEL = BSP_CFG_MAIN_CLOCK_SOURCE;

        /* Set the oscillation stabilization wait time of the main clock oscillator. */
        #if (BSP_CFG_MAIN_CLOCK_SOURCE == 0) /* Resonator */
            SYSTEM.MOSCWTCR.BYTE = BSP_CFG_MOSC_WAIT_TIME;
        #elif (BSP_CFG_MAIN_CLOCK_SOURCE == 1) /* External oscillator input */
            SYSTEM.MOSCWTCR.BYTE = 0x00;
        #else
            #error "Error! Invalid setting for BSP_CFG_MAIN_CLOCK_SOURCE in r_bsp_config.h"
        #endif 

        /* Set the main clock to operating. */
        SYSTEM.MOSCCR.BYTE = 0x00;

        /* Set PLL Input Divisor. */
        SYSTEM.PLLCR.BIT.PLIDIV = BSP_CFG_PLL_DIV >> 1;

        /* Set PLL Multiplier. */
        SYSTEM.PLLCR.BIT.STC = (BSP_CFG_PLL_MUL * 2) - 1;

        /* Set the PLL to operating. */
        SYSTEM.PLLCR2.BYTE = 0x00;
    }

    /* WAIT_LOOP */
    while (0 == SYSTEM.OSCOVFSR.BIT.PLOVF)
    {
        /* Make sure clock has stabilized. */
    }
#else
    /* Set the PLL to stopped. */
    SYSTEM.PLLCR2.BYTE = 0x01;
#endif

    /* LOCO is saved for last since it is what is running by default out of reset. This means you do not want to turn
       it off until another clock has been enabled and is ready to use. */
#if (BSP_CFG_CLOCK_SOURCE == 0)
    /* LOCO is chosen. This is the default out of reset. */
    SYSTEM.LOCOCR.BYTE = 0x00;
#else
    /* LOCO is not chosen but it cannot be turned off yet since it is still being used. */
#endif

    /* Make sure a valid clock was chosen. */
#if (BSP_CFG_CLOCK_SOURCE > 4) || (BSP_CFG_CLOCK_SOURCE < 0)
    #error "ERROR - Valid clock source must be chosen in r_bsp_config.h using BSP_CFG_CLOCK_SOURCE macro."
#endif
}

/***********************************************************************************************************************
* Function name: lpt_clock_source_select
* Description  : Enables clock sources for the lpt (if not already done) as chosen by the user.
* Arguments    : none
* Return value : none
***********************************************************************************************************************/
static void lpt_clock_source_select (void)
{
    /* Declared volatile for software delay purposes. */
    volatile uint32_t i;


    /* Protect off. */
    SYSTEM.PRCR.WORD = 0xA50F;


    /* INITIALIZE AND SELECT LPT CLOCK SOURCE */

#if (BSP_CFG_LPT_CLOCK_SOURCE == 0) || (BSP_CFG_LPT_CLOCK_SOURCE == 2)
    /* Sub-clock or None is chosen. */
    /* sub-clock oscillator already initialized in clock_source_select() */

#elif (BSP_CFG_LPT_CLOCK_SOURCE == 1)
    /* IWDTCLK is chosen. Start it operating. */
    SYSTEM.ILOCOCR.BYTE = 0x00;

    /* Wait processing for the IWDT clock oscillation stabilization (50us) */
    /* WAIT_LOOP */
    for(i = 0; i < IWDTCLK_STABILIZE_LOOP_CNT; i++)
    {
        nop() ;                     /* wait for stabilization */
    }

    /* Controls whether to stop the IWDT counter in a low power consumption state.
    IWDTCSTPR - IWDT Count Stop Control Register
    b7     SLCSTP   - Sleep Mode Count Stop Control - Count stop is disabled.
    b6:b1  Reserved - These bits are read as 0. Writing to these bits has no effect. */
    IWDT.IWDTCSTPR.BIT.SLCSTP = 0;

#else
    #error "Error! Invalid setting for BSP_CFG_LPT_CLOCK_SOURCE in r_bsp_config.h"
#endif

    SYSTEM.PRCR.WORD = 0xA500;
    return;
}

#endif /* BSP_CFG_STARTUP_DISABLE == 0 */
